"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const form_1 = require("../form");
const controls_1 = require("./models/controls");
const formControl_1 = require("../formControl");
let _controls;
let _dependentControls;
describe('Form', () => {
    beforeEach(() => {
        _controls = controls_1.controls;
        _dependentControls = controls_1.dependenControls;
    });
    test('Create Form function - invalid form with invalid property value', () => {
        const form = new form_1.Form(_controls);
        expect(form).toBeTruthy();
        const formcontrols = form.controls;
        expect(formcontrols.test.error).toBeTruthy();
        expect(formcontrols.test.errorMessage).toBe('test is required');
        expect(form.valid).toBeFalsy();
    });
    test('Create Form function - valid form with valid property value', () => {
        const form = new form_1.Form({
            test: new formControl_1.FormControl(null, 'create form test', [
                {
                    validatorfunction: controls_1.hasValue,
                    errorMessage: 'test is required',
                },
            ]),
        });
        expect(form).toBeTruthy();
        const formcontrols = form.controls;
        expect(formcontrols.test.error).toBeFalsy();
        expect(formcontrols.test.errorMessage).toBe('');
        expect(form.valid).toBeTruthy();
    });
    test('Create Form function - validation with dependent controls', () => {
        const form = new form_1.Form(_dependentControls);
        expect(form).toBeTruthy();
        const formcontrols = form.controls;
        expect(formcontrols.control2.error).toBeTruthy();
        expect(formcontrols.control2.errorMessage).toBe('Control2 should match with control1');
        expect(form.valid).toBeFalsy();
    });
    test('Create Form function - non arguments', () => {
        const form = new form_1.Form();
        expect(form).toMatchObject({ controls: {}, valid: true });
    });
    test('Should return form value', () => {
        const form = new form_1.Form(_controls);
        expect(form.value).toMatchObject({ test: '' });
    });
});
