"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_hooks_1 = require("@testing-library/react-hooks");
const __1 = require("..");
const controls_1 = require("./models/controls");
const formControl_1 = require("../formControl");
test('Should use useForm hook', () => {
    const { result } = react_hooks_1.renderHook(() => __1.useForm(controls_1.controls));
    expect(result.current.form.controls.test.value).toBe('');
    expect(result.current.form.controls.test.error).toBeTruthy();
    expect(result.current.form.controls.test.errorMessage).toBe('test is required');
    expect(result.current.form.valid).toBeFalsy();
});
test('Should update form state - on handleControlEvent', () => {
    const { result } = react_hooks_1.renderHook(() => __1.useForm(controls_1.controls));
    react_hooks_1.act(() => {
        result.current.handleControlEvent({
            target: { name: 'test', value: 'new test value' },
        });
    });
    expect(result.current.form.controls.test.value).toEqual('new test value');
    expect(result.current.form.controls.test.error).toBeFalsy();
    expect(result.current.form.controls.test.errorMessage).toBe('');
    expect(result.current.form.valid).toBeTruthy();
});
test('Should use useForm hook -  dependent controls', () => {
    const { result } = react_hooks_1.renderHook(() => __1.useForm(controls_1.dependenControls));
    expect(result.current.form.controls.control2.value).toEqual('');
    expect(result.current.form.controls.control2.error).toBeTruthy();
    expect(result.current.form.controls.control2.errorMessage).toBe('Control2 should match with control1');
    expect(result.current.form.valid).toBeFalsy();
});
test('Should update form state - on handleControlEvent', () => {
    const { result } = react_hooks_1.renderHook(() => __1.useForm(controls_1.dependenControls));
    react_hooks_1.act(() => {
        result.current.handleControlEvent({
            target: { name: 'control2', value: 'should match' },
        });
    });
    expect(result.current.form.controls.control2.value).toEqual('should match');
    expect(result.current.form.controls.control2.error).toBeFalsy();
    expect(result.current.form.controls.control2.errorMessage).toBe('');
    expect(result.current.form.valid).toBeTruthy();
});
test('Should reset form state', () => {
    const { result } = react_hooks_1.renderHook(() => __1.useForm(controls_1.dependenControls));
    expect(result.current.form.controls).toMatchObject(controls_1.dependenControls);
    expect(result.current.form.valid).toBeFalsy();
    react_hooks_1.act(() => {
        result.current.handleControlEvent({
            target: { name: 'control2', value: 'should match' },
        });
    });
    expect(result.current.form.controls.control2.value).toEqual('should match');
    expect(result.current.form.valid).toBeTruthy();
    react_hooks_1.act(() => {
        result.current.resetForm();
    });
    expect(result.current.form.controls).toMatchObject(controls_1.dependenControls);
    expect(result.current.form.valid).toBeFalsy();
});
test('Should add new form control', () => {
    const { result } = react_hooks_1.renderHook(() => __1.useForm(controls_1.controls));
    expect(result.current.form.controls).toMatchObject(controls_1.controls);
    expect(result.current.form.valid).toBeFalsy();
    react_hooks_1.act(() => {
        result.current.handleControlEvent({
            target: { name: 'test', value: 'Form Test' },
        });
    });
    expect(result.current.form.valid).toBeTruthy();
    react_hooks_1.act(() => {
        result.current.addFormControl('coverage', new formControl_1.FormControl(null, '', [
            {
                validatorfunction: (val) => val !== '',
                errorMessage: 'coverage is required',
            },
        ]));
    });
    expect(result.current.form.valid).toBeFalsy();
});
test('Should remove form control', () => {
    const { result } = react_hooks_1.renderHook(() => __1.useForm(controls_1.dependenControls));
    expect(result.current.form.controls).toMatchObject(controls_1.dependenControls);
    expect(result.current.form.valid).toBeFalsy();
    react_hooks_1.act(() => {
        result.current.handleControlEvent({
            target: { name: 'control2', value: 'should match' },
        });
    });
    expect(result.current.form.valid).toBeTruthy();
    react_hooks_1.act(() => {
        result.current.removeFormControl('control2');
    });
    expect(result.current.form.controls).toMatchObject({
        control1: {
            value: 'should match',
            error: false,
            errorMessage: '',
            validators: [
                {
                    validatorfunction: controls_1.hasValue,
                    errorMessage: 'Control1 is required',
                },
            ],
        },
    });
});
