"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const formControl_1 = require("./formControl");
class Form {
    constructor(_controls = {}) {
        this.formatControls = (formControls = {}) => {
            const controls = Object.create({});
            try {
                Object.keys(formControls).forEach((key) => {
                    const { value, validators = [] } = formControls[key];
                    const control = new formControl_1.FormControl(formControls, value, validators);
                    controls[key] = control;
                });
                return controls;
            }
            catch (error) {
                throw new Error(`Invalid form format: ${error.message}`);
            }
        };
        this._controls = this.formatControls(_controls);
    }
    get controls() {
        return this._controls;
    }
    get valid() {
        return this.isValidForm();
    }
    get value() {
        return this.getValues();
    }
    isValidForm() {
        const isValid = !Object.keys(this.controls).find((prop) => this.controls[prop].validators.some((validator) => validator.validatorfunction.call(this.controls, this.controls[prop].value) === false));
        return isValid;
    }
    getValues() {
        return Object.keys(this.controls).reduce((obj, prop) => {
            obj[prop] = this.controls[prop].value;
            return obj;
        }, {});
    }
}
exports.Form = Form;
