"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createForm = (formControls = {}) => {
    const controls = Object.create({});
    try {
        Object.keys(formControls).forEach((key) => {
            const formControl = formControls[key];
            const { value, validators = [], error = false, errorMessage = '' } = formControl;
            const control = { value, validators, error, errorMessage };
            for (const validator of control.validators) {
                if (!validator.validatorfunction.call(controls, control.value)) {
                    control.error = true;
                    control.errorMessage = validator.errorMessage;
                    break;
                }
            }
            controls[key] = control;
        });
        return {
            controls,
            valid: isValidForm(controls),
        };
    }
    catch (error) {
        throw new Error(`Invalid form format: ${error.message}`);
    }
};
function isValidForm(controls) {
    const isValid = !Object.keys(controls).find((prop) => controls[prop].validators.some((validator) => validator.validatorfunction.call(controls, controls[prop].value) === false));
    return isValid;
}
exports.isValidForm = isValidForm;
