"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const formControl_1 = require("./formControl");
const react_1 = require("react");
const form_1 = require("./form");
exports.useForm = (controls) => {
    const [form, setForm] = react_1.useState(new form_1.Form(controls));
    const updateForm = (formControls) => {
        setForm((prevState) => {
            const updatedControls = Object.assign(Object.assign({}, prevState.controls), formControls);
            return new form_1.Form(updatedControls);
        });
    };
    const handleControlEvent = (event) => {
        const { value, name, type, checked } = event.target;
        if (name) {
            const validators = form.controls[name].validators;
            const _value = type === 'checkbox' ? checked : value;
            const control = new formControl_1.FormControl(null, _value, validators, true);
            updateForm({ [name]: control });
        }
        else {
            throw new Error('Missing property name. <input *name="prop_name"/>');
        }
    };
    const setFormControlValue = (key, value) => {
        const validators = form.controls[key].validators;
        updateForm({ [key]: new formControl_1.FormControl(null, value, validators, true) });
    };
    const resetForm = () => {
        updateForm(Object.assign({}, controls));
    };
    const addFormControl = (name, control) => {
        updateForm({ [name]: control });
    };
    const removeFormControl = (name) => {
        const currentControls = Object.assign({}, form.controls);
        delete currentControls[name];
        try {
            setForm(new form_1.Form(currentControls));
        }
        catch (error) {
            throw new Error(`Unable to remove control. Maybe has dependency relation with other controls: ${error}`);
        }
    };
    return {
        form,
        handleControlEvent,
        setFormControlValue,
        resetForm,
        addFormControl,
        removeFormControl,
    };
};
